@@ -64,6 +64,10 @@ class VNFCreateFailed(exceptions.TackerException):
     message = _('creating VNF based on %(vnfd_id)s failed')
 
 
+class VNFUpdateInvalidInput(exceptions.TackerException):
+    message = _('VNF Update Invalid Input %(reason)s')
+
+
 class VNFUpdateWaitFailed(exceptions.TackerException):
     message = _('VNF Update %(reason)s')
 
@@ -104,6 +108,10 @@ class VNFNotFound(exceptions.NotFound):
     message = _('VNF %(vnf_id)s could not be found')
 
 
+class LCMUserDataFailed(exceptions.TackerException):
+    message = _('LCM user data %(reason)s')
+
+
 class ParamYAMLNotWellFormed(exceptions.InvalidInput):
     message = _("Parameter YAML not well formed - %(error_msg_details)s")
 
@@ -195,6 +203,11 @@ class InvalidKubernetesInputParameter(exceptions.InvalidInput):
     message = _("Found unsupported keys for %(found_keys)s ")
 
 
+class InvalidInstReqInfoForScaling(exceptions.InvalidInput):
+    message = _("Scaling resource cannot be set to "
+                "fixed ip_address or mac_address.")
+
+
 def _validate_service_type_list(data, valid_values=None):
     if not isinstance(data, list):
         msg = _("Invalid data format for service list: '%s'") % data
