@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import base64
 import os
 
 import glance_store
@@ -41,20 +42,24 @@ def get_csar_data_iter(body):
     try:
         if isinstance(body, dict):
             url = body['address_information']
-            data_iter = urllib.request.urlopen(url)
+            req = urllib.request.Request(url)
+            if body['user_name'] is not None or body['password'] is not None:
+                _add_basic_auth(req, body['user_name'], body['password'])
+            data_iter = urllib.request.urlopen(req)
         else:
             data_iter = body
 
         return data_iter
-    except Exception:
-        LOG.warn("Failed to open csar URL: %s", url)
+    except Exception as e:
+        error = encodeutils.exception_to_unicode(e)
+        LOG.warn("Failed to open csar URL: %(url)s due to error: %(error)s",
+                 {"url": url, "error": error})
         raise exceptions.VNFPackageURLInvalid(url=url)
 
 
 def store_csar(context, package_uuid, body):
 
     data_iter = get_csar_data_iter(body)
-
     try:
         # store CSAR file in glance_store
         (location, size, checksum, multihash,
@@ -91,16 +96,21 @@ def delete_csar(context, package_uuid, location):
                  {"uuid": package_uuid})
 
 
+def get_csar_size(package_uuid, location):
+
+    try:
+        return glance_store.backend.get_size_from_backend(location)
+    except Exception:
+        LOG.exception("Failed to get csar data from glance store %(location)s"
+         "for package %(uuid)s", {"location": location, "uuid": package_uuid})
+        raise exceptions.VnfPackageLocationInvalid(location=location)
+
+
 def load_csar(package_uuid, location):
     zip_path = os.path.join(CONF.vnf_package.vnf_package_csar_path,
                             package_uuid + ".zip")
-
-    try:
-        resp, size = glance_store.backend.get_from_backend(location)
-    except Exception as exp:
-        LOG.info("Failed to get csar data from glance store %(location)s for "
-                 "package %(uuid)s",
-                 {"location": location, "uuid": package_uuid})
+    resp, size = _get_csar_chunks(
+        package_uuid, location, offset=0, chunk_size=None)
 
     try:
         temp_data = open(zip_path, 'wb')
@@ -115,3 +125,32 @@ def load_csar(package_uuid, location):
                        'error': encodeutils.exception_to_unicode(exp)})
 
     return zip_path
+
+
+def load_csar_iter(package_uuid, location, offset=0, chunk_size=None):
+    resp, size = _get_csar_chunks(
+        package_uuid, location, offset=offset, chunk_size=chunk_size)
+    return resp, size
+
+
+def _get_csar_chunks(package_uuid, location, offset, chunk_size):
+    try:
+        resp, size = glance_store.backend.get_from_backend(location,
+                                                       offset=offset,
+                                                       chunk_size=chunk_size)
+        return resp, size
+    except Exception:
+        LOG.exception("Failed to get csar data from glance store %(location)s"
+         "for package %(uuid)s", {"location": location, "uuid": package_uuid})
+        raise exceptions.VnfPackageLocationInvalid(location=location)
+
+
+def _add_basic_auth(request, username, password):
+    """A helper function to add basic authentication.
+
+    This function adds basic authentication information to a six.moves.urllib
+    request.
+    """
+    auth_str = base64.encodestring(('%s:%s' % (
+        username, password)).encode()).decode().strip()
+    request.add_header('Authorization', 'Basic %s' % auth_str)
