@@ -25,3 +25,13 @@ def register_all():
     # function in order for it to be registered by services that may
     # need to receive it via RPC.
     __import__('tacker.objects.heal_vnf_request')
+    __import__('tacker.objects.vnf_package')
+    __import__('tacker.objects.vnf_package_vnfd')
+    __import__('tacker.objects.vnf_deployment_flavour')
+    __import__('tacker.objects.vnf_software_image')
+    __import__('tacker.objects.vnf_instance')
+    __import__('tacker.objects.vnf_instantiated_info')
+    __import__('tacker.objects.vim_connection')
+    __import__('tacker.objects.instantiate_vnf_req')
+    __import__('tacker.objects.vnf_resources')
+    __import__('tacker.objects.terminate_vnf_req')
