@@ -12,6 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import yaml
+
 from tacker.plugins.common import constants as evt_constants
 from tacker.tests import constants
 from tacker.tests.functional import base
@@ -20,12 +22,11 @@ from tacker.tests.utils import read_file
 
 class VnfTestPingMonitor(base.BaseTackerTest):
 
-    def _test_vnf_with_monitoring(self, vnfd_file, vnf_name):
-        data = dict()
-        data['tosca'] = read_file(vnfd_file)
-        toscal = data['tosca']
+    def _vnfd_and_vnf_create(self, vnfd_file, vnf_name):
+        input_yaml = read_file(vnfd_file)
+        tosca_dict = yaml.safe_load(input_yaml)
         tosca_arg = {'vnfd': {'name': vnf_name,
-                              'attributes': {'vnfd': toscal}}}
+                              'attributes': {'vnfd': tosca_dict}}}
 
         # Create vnfd with tosca template
         vnfd_instance = self.client.create_vnfd(body=tosca_arg)
@@ -36,6 +37,15 @@ class VnfTestPingMonitor(base.BaseTackerTest):
         vnf_arg = {'vnf': {'vnfd_id': vnfd_id, 'name': vnf_name}}
         vnf_instance = self.client.create_vnf(body=vnf_arg)
 
+        # Delete vnfd_instance
+        self.addCleanup(self.client.delete_vnfd, vnfd_id)
+
+        return vnfd_instance, vnf_instance
+
+    def _test_vnf_with_monitoring(self, vnfd_file, vnf_name):
+        vnfd_instance, vnf_instance = self._vnfd_and_vnf_create(vnfd_file,
+                                                                vnf_name)
+
         # Verify vnf goes from ACTIVE->DEAD->ACTIVE states
         self.verify_vnf_restart(vnfd_instance, vnf_instance)
 
@@ -51,31 +61,9 @@ class VnfTestPingMonitor(base.BaseTackerTest):
         vnf_state_list = [evt_constants.ACTIVE, evt_constants.DEAD]
         self.verify_vnf_monitor_events(vnf_id, vnf_state_list)
 
-        # Delete vnfd_instance
-        self.addCleanup(self.client.delete_vnfd, vnfd_id)
         self.addCleanup(self.wait_until_vnf_delete, vnf_id,
             constants.VNF_CIRROS_DELETE_TIMEOUT)
 
-    def test_create_delete_vnf_monitoring(self):
-        self._test_vnf_with_monitoring(
-            'sample-vnfd-single-vdu-monitoring.yaml',
-            'ping monitor vnf')
-
-    def test_create_delete_vnf_http_monitoring(self):
-        self._test_vnf_with_monitoring(
-            'sample_cirros_http_monitoring.yaml',
-            'http monitor vnf')
-
-    def test_create_delete_vnf_multi_vdu_ping_monitoring(self):
-        self._test_vnf_with_monitoring(
-            'sample-vnfd-multi-vdu-monitoring.yaml',
-            'multi vdu ping monitor vnf')
-
-    def test_create_delete_vnf_monitoring_new_template(self):
-        self._test_vnf_with_monitoring(
-            'sample-vnfd-single-vdu-monitoring-new-template.yaml',
-            'ping monitor vnf new template')
-
     def test_create_delete_vnf_monitoring_tosca_template(self):
         self._test_vnf_with_monitoring(
             'sample-tosca-vnfd-monitor.yaml',
@@ -86,7 +74,38 @@ class VnfTestPingMonitor(base.BaseTackerTest):
             'sample-tosca-vnfd-multi-vdu-monitoring.yaml',
             'ping monitor multi vdu vnf with tosca template')
 
-    def test_create_delete_vnf_http_monitoring_tosca_template(self):
-        self._test_vnf_with_monitoring(
-            'sample-tosca-vnfd-http-monitor.yaml',
-            'http monitor vnf with tosca template')
+    def _test_vnf_with_monitoring_vdu_autoheal_action(
+            self, vnfd_file, vnf_name):
+        vnfd_instance, vnf_instance = self._vnfd_and_vnf_create(vnfd_file,
+                                                                vnf_name)
+        vnf_id = vnf_instance['vnf']['id']
+
+        self.verify_vnf_update(vnf_id)
+
+        # Delete vnf_instance with vnf_id
+        try:
+            self.client.delete_vnf(vnf_id)
+        except Exception:
+            assert False, ("Failed to delete vnf %s after the monitor test" %
+                           vnf_id)
+        self.addCleanup(self.wait_until_vnf_delete, vnf_id,
+                        constants.VNF_CIRROS_DELETE_TIMEOUT)
+
+        params = {'resource_id': vnf_id,
+                  'resource_state': 'PENDING_UPDATE',
+                  'event_type': evt_constants.RES_EVT_MONITOR}
+        vnf_events = self.client.list_vnf_events(**params)
+        # Check if vdu_autoheal action emits 4 monitoring events.
+        self.assertGreaterEqual(4, len(vnf_events['vnf_events']))
+
+    def test_vnf_monitoring_with_vdu_autoheal_action_for_multi_vdu(self):
+        self._test_vnf_with_monitoring_vdu_autoheal_action(
+            'sample-tosca-vnfd-multi-vdu-monitoring-vdu-autoheal.yaml',
+            'ping multi vdu monitor having vdu_autoheal failure action '
+            'with tosca template')
+
+    def test_vnf_monitoring_with_vdu_autoheal_action_for_single_vdu(self):
+        self._test_vnf_with_monitoring_vdu_autoheal_action(
+            'sample-tosca-vnfd-single-vdu-monitoring-vdu-autoheal.yaml',
+            'ping vdu monitor having vdu_autoheal failure action '
+            'with tosca template')
