@@ -13,10 +13,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import os
+
 from keystoneauth1 import exceptions
 import mock
 from oslo_config import cfg
 
+from tacker import context as t_context
 from tacker.extensions import nfvo
 from tacker.nfvo.drivers.vim import openstack_driver
 from tacker.tests.unit import base
@@ -28,8 +31,23 @@ OPTS = [cfg.StrOpt('user_domain_id',
         cfg.StrOpt('project_domain_id',
                    default='default',
                    help='Project Domain Id'),
+        cfg.StrOpt('password',
+                   default='default',
+                   help='User Password'),
+        cfg.StrOpt('username',
+                   default='default',
+                   help='User Name'),
+        cfg.StrOpt('user_domain_name',
+                   default='default',
+                   help='Use Domain Name'),
+        cfg.StrOpt('project_name',
+                   default='default',
+                   help='Project Name'),
+        cfg.StrOpt('project_domain_name',
+                   default='default',
+                   help='Project Domain Name'),
         cfg.StrOpt('auth_url',
-                   default='http://localhost:5000/v3',
+                   default='http://localhost/identity/v3',
                    help='Keystone endpoint')]
 
 cfg.CONF.register_opts(OPTS, 'keystone_authtoken')
@@ -86,24 +104,26 @@ class TestOpenstack_Driver(base.TestCase):
 
     def get_vim_obj(self):
         return {'id': '6261579e-d6f3-49ad-8bc3-a9cb974778ff', 'type':
-                'openstack', 'auth_url': 'http://localhost:5000',
+                'openstack', 'auth_url': 'http://localhost/identity',
                 'auth_cred': {'username': 'test_user',
                               'password': 'test_password',
                               'user_domain_name': 'default',
-                              'auth_url': 'http://localhost:5000'},
+                              'cert_verify': 'True',
+                              'auth_url': 'http://localhost/identity'},
                 'name': 'VIM0',
                 'vim_project': {'name': 'test_project',
                                 'project_domain_name': 'default'}}
 
     def get_vim_obj_barbican(self):
         return {'id': '6261579e-d6f3-49ad-8bc3-a9cb974778ff', 'type':
-                'openstack', 'auth_url': 'http://localhost:5000',
+                'openstack', 'auth_url': 'http://localhost/identity',
                 'auth_cred': {'username': 'test_user',
                               'password': 'test_password',
                               'user_domain_name': 'default',
                               'key_type': 'barbican_key',
                               'secret_uuid': 'fake-secret-uuid',
-                              'auth_url': 'http://localhost:5000'},
+                              'cert_verify': 'True',
+                              'auth_url': 'http://localhost/identity'},
                 'name': 'VIM0',
                 'vim_project': {'name': 'test_project',
                                 'project_domain_name': 'default'}}
@@ -112,43 +132,26 @@ class TestOpenstack_Driver(base.TestCase):
         regions = [mock_dict({'id': 'RegionOne'})]
         attrs = {'regions.list.return_value': regions}
         keystone_version = 'v3'
-        mock_ks_client = mock.Mock(version=keystone_version, **attrs)
+        mock_ks_client = mock.Mock(**attrs)
         self.keystone.get_version.return_value = keystone_version
         self._test_register_vim(self.vim_obj, mock_ks_client)
         mock_ks_client.regions.list.assert_called_once_with()
         self.keystone.initialize_client.assert_called_once_with(
-            version=keystone_version, **self.auth_obj)
-
-    def test_register_keystone_v2(self):
-        services_list = [mock_dict({'type': 'orchestration', 'id':
-                         'test_id'})]
-        endpoints_regions = mock_dict({'region': 'RegionOne'})
-        endpoints_list = [mock_dict({'service_id': 'test_id', 'regions':
-                          endpoints_regions})]
-        attrs = {'endpoints.list.return_value': endpoints_list,
-                 'services.list.return_value': services_list}
-        keystone_version = 'v2.0'
-        mock_ks_client = mock.Mock(version='v2.0', **attrs)
-        self.keystone.get_version.return_value = keystone_version
-        auth_obj = {'tenant_name': 'test_project', 'username': 'test_user',
-                    'password': 'test_password', 'auth_url':
-                    'http://localhost:5000/v2.0', 'tenant_id': None}
-        self._test_register_vim(self.vim_obj, mock_ks_client)
-        self.keystone.initialize_client.assert_called_once_with(
-            version=keystone_version, **auth_obj)
+            **self.auth_obj)
 
     def _test_register_vim(self, vim_obj, mock_ks_client):
         self.keystone.initialize_client.return_value = mock_ks_client
         fernet_attrs = {'encrypt.return_value': 'encrypted_password'}
         mock_fernet_obj = mock.Mock(**fernet_attrs)
-        mock_fernet_key = 'test_fernet_key'
+        mock_fernet_key = b'test_fernet_key'
         self.keystone.create_fernet_key.return_value = (mock_fernet_key,
                                                         mock_fernet_obj)
-        file_mock = mock.mock_open()
-        with mock.patch('six.moves.builtins.open', file_mock, create=True):
-            self.openstack_driver.register_vim(None, vim_obj)
+        self.openstack_driver.register_vim(vim_obj)
+        with open('/tmp/' + vim_obj['id'], 'r') as f:
+            # asserting that file has been written correctly.
+            self.assertEqual('test_fernet_key', f.read())
         mock_fernet_obj.encrypt.assert_called_once_with(mock.ANY)
-        file_mock().write.assert_called_once_with('test_fernet_key')
+        os.remove('/tmp/' + vim_obj['id'])
 
     @mock.patch('tacker.nfvo.drivers.vim.openstack_driver.os.remove')
     @mock.patch('tacker.nfvo.drivers.vim.openstack_driver.os.path'
@@ -159,15 +162,15 @@ class TestOpenstack_Driver(base.TestCase):
         vim_obj['id'] = vim_id
         file_path = CONF.vim_keys.openstack + '/' + vim_id
         mock_os_path.return_value = file_path
-        self.openstack_driver.deregister_vim(None, vim_obj)
+        self.openstack_driver.deregister_vim(vim_obj)
         mock_os_remove.assert_called_once_with(file_path)
 
     def test_deregister_vim_barbican(self):
         self.keymgr.delete.return_value = None
         vim_obj = self.get_vim_obj_barbican()
-        self.openstack_driver.deregister_vim(None, vim_obj)
+        self.openstack_driver.deregister_vim(vim_obj)
         self.keymgr.delete.assert_called_once_with(
-            None, 'fake-secret-uuid')
+            t_context.generate_tacker_service_context(), 'fake-secret-uuid')
 
     def test_encode_vim_auth_barbican(self):
         self.config_fixture.config(group='vim_keys',
@@ -181,10 +184,10 @@ class TestOpenstack_Driver(base.TestCase):
 
         vim_obj = self.get_vim_obj()
         self.openstack_driver.encode_vim_auth(
-            None, vim_obj['id'], vim_obj['auth_cred'])
+            vim_obj['id'], vim_obj['auth_cred'])
 
         self.keymgr.store.assert_called_once_with(
-            None, 'test_fernet_key')
+            t_context.generate_tacker_service_context(), 'test_fernet_key')
         mock_fernet_obj.encrypt.assert_called_once_with(mock.ANY)
         self.assertEqual(vim_obj['auth_cred']['key_type'],
                          'barbican_key')
@@ -201,16 +204,15 @@ class TestOpenstack_Driver(base.TestCase):
 
     def _test_register_vim_auth(self, attrs):
         keystone_version = 'v3'
-        mock_ks_client = mock.Mock(version=keystone_version, **attrs)
         self.keystone.get_version.return_value = keystone_version
+        mock_ks_client = mock.Mock(**attrs)
         self.keystone.initialize_client.return_value = mock_ks_client
         self.assertRaises(nfvo.VimUnauthorizedException,
                           self.openstack_driver.register_vim,
-                          None,
                           self.vim_obj)
         mock_ks_client.regions.list.assert_called_once_with()
         self.keystone.initialize_client.assert_called_once_with(
-            version=keystone_version, **self.auth_obj)
+            **self.auth_obj)
 
     def test_get_vim_resource_id(self):
         resource_type = 'network'
