@@ -14,18 +14,72 @@
 #    under the License.
 
 import codecs
-import json
-import mock
 import os
+
+import mock
 import yaml
 
+from oslo_serialization import jsonutils
+
 from tacker import context
+from tacker.db.common_services import common_services_db_plugin
 from tacker.extensions import vnfm
+from tacker.tests.common import helpers
 from tacker.tests.unit import base
 from tacker.tests.unit.db import utils
 from tacker.vnfm.infra_drivers.openstack import openstack
 
 
+vnf_dict = {
+    'attributes': {
+        'heat_template': {
+            'outputs': {
+                'mgmt_ip-VDU1': {
+                    'value': {
+                        'get_attr': [
+                            'CP1', 'fixed_ips', 0, 'ip_address']
+                    }
+                }
+            },
+            'description': 'Demo example\n',
+            'parameters': {},
+            'resources': {
+                'VDU1': {
+                    'type': 'OS::Nova::Server',
+                    'properties': {
+                        'user_data_format': 'SOFTWARE_CONFIG',
+                        'availability_zone': 'nova',
+                        'image': 'cirros-0.4.0-x86_64-disk',
+                        'config_drive': False,
+                        'flavor': {'get_resource': 'VDU1_flavor'},
+                        'networks': [{'port': {'get_resource': 'CP1'}}]
+                    }
+                },
+                'CP1': {
+                    'type': 'OS::Neutron::Port',
+                    'properties': {
+                        'port_security_enabled': False,
+                        'network': 'net_mgmt'
+                    }
+                },
+                'VDU1_flavor': {
+                    'type': 'OS::Nova::Flavor',
+                    'properties': {'vcpus': 1, 'disk': 1, 'ram': 512}
+                }
+            }
+        }
+    },
+    'status': 'ACTIVE',
+    'vnfd_id': '576acf48-b9df-491d-a57c-342de660ec78',
+    'tenant_id': '13d2ca8de70d48b2a2e0dbac2c327c0b',
+    'vim_id': '3f41faa7-5630-47d2-9d4a-1216953c8887',
+    'instance_id': 'd1121d3c-368b-4ac2-b39d-835aa3e4ccd8',
+    'placement_attr': {'vim_name': 'openstack-vim'},
+    'id': 'a27fc58e-66ae-4031-bba4-efede318c60b',
+    'name': 'vnf_create_1'
+}
+
+
 class FakeHeatClient(mock.Mock):
 
     class Stack(mock.Mock):
@@ -43,8 +97,8 @@ class FakeHeatClient(mock.Mock):
 def _get_template(name):
     filename = os.path.join(
         os.path.dirname(os.path.abspath(__file__)), "data/", name)
-    f = codecs.open(filename, encoding='utf-8', errors='strict')
-    return f.read()
+    with codecs.open(filename, encoding='utf-8', errors='strict') as f:
+        return f.read()
 
 
 class TestOpenStack(base.TestCase):
@@ -52,6 +106,7 @@ class TestOpenStack(base.TestCase):
     hot_param_template = _get_template('hot_openwrt_params.yaml')
     hot_ipparam_template = _get_template('hot_openwrt_ipparams.yaml')
     tosca_vnfd_openwrt = _get_template('test_tosca_openwrt.yaml')
+    tosca_vnfd_openwrt_param = _get_template('test_tosca_openwrt_param.yaml')
     config_data = _get_template('config_data.yaml')
 
     def setUp(self):
@@ -59,7 +114,15 @@ class TestOpenStack(base.TestCase):
         self.context = context.get_admin_context()
         self.infra_driver = openstack.OpenStack()
         self._mock_heat_client()
+        mock.patch('tacker.db.common_services.common_services_db_plugin.'
+                   'CommonServicesPluginDb.create_event'
+                   ).start()
+        self._cos_db_plugin = \
+            common_services_db_plugin.CommonServicesPluginDb()
         self.addCleanup(mock.patch.stopall)
+        yaml.SafeLoader.add_constructor(
+            yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
+            lambda loader, node: dict(loader.construct_pairs(node)))
 
     def _mock_heat_client(self):
         self.heat_client = mock.Mock(wraps=FakeHeatClient())
@@ -115,7 +178,7 @@ class TestOpenStack(base.TestCase):
                     'attributes': {u'vnfd': self.tosca_vnfd_openwrt},
                     'id': u'fb048660-dc1b-4f0f-bd89-b023666650ec',
                     'name': u'OpenWRT'},
-                'mgmt_url': '{"vdu1": "192.168.120.31"}',
+                'mgmt_ip_address': '{"vdu1": "192.168.120.31"}',
                 'service_context': [],
                 'attributes': {u'param_values': param_values},
                 'id': 'eb84260e-5ff7-4332-b032-50a14d6c1123',
@@ -132,11 +195,46 @@ class TestOpenStack(base.TestCase):
             u'ad7ebc56538745a08ef7c5e97f8bd437', 'mgmt_driver': u'openwrt',
             'attributes': {u'vnfd': self.tosca_vnfd_openwrt},
             'id': u'fb048660-dc1b-4f0f-bd89-b023666650ec', 'name':
-            u'openwrt_services'}, 'mgmt_url': None, 'service_context': [],
+            u'openwrt_services'}, 'mgmt_ip_address': None,
+            'service_context': [],
             'attributes': {'config': utils.update_config_data},
             'id': 'eb84260e-5ff7-4332-b032-50a14d6c1123', 'description':
                 u'OpenWRT with services'}
 
+    def _get_expected_vnf_update_param_obj(self):
+        return {'status': 'PENDING_CREATE', 'instance_id': None, 'name':
+            u'test_openwrt', 'tenant_id':
+        u'ad7ebc56538745a08ef7c5e97f8bd437', 'vnfd_id':
+        u'eb094833-995e-49f0-a047-dfb56aaf7c4e', 'vnfd': {
+            'service_types': [{'service_type': u'vnfd', 'id':
+            u'4a4c2d44-8a52-4895-9a75-9d1c76c3e738'}], 'description':
+            u'OpenWRT with services', 'tenant_id':
+            u'ad7ebc56538745a08ef7c5e97f8bd437', 'mgmt_driver': u'openwrt',
+            'attributes': {u'vnfd': self.tosca_vnfd_openwrt_param},
+            'id': u'fb048660-dc1b-4f0f-bd89-b023666650ec', 'name':
+            u'openwrt_services'}, 'mgmt_url': None, 'service_context': [],
+            'attributes': {'heat_template': utils.hot_data,
+                           'param_values': utils.update_param_data},
+            'id': 'eb84260e-5ff7-4332-b032-50a14d6c1123', 'description':
+                u'OpenWRT with services'}
+
+    def _get_expected_vnf_update_new_param_obj(self):
+        return {'status': 'PENDING_CREATE', 'instance_id': None, 'name':
+            u'test_openwrt', 'tenant_id':
+        u'ad7ebc56538745a08ef7c5e97f8bd437', 'vnfd_id':
+        u'eb094833-995e-49f0-a047-dfb56aaf7c4e', 'vnfd': {
+            'service_types': [{'service_type': u'vnfd', 'id':
+            u'4a4c2d44-8a52-4895-9a75-9d1c76c3e738'}], 'description':
+            u'OpenWRT with services', 'tenant_id':
+            u'ad7ebc56538745a08ef7c5e97f8bd437', 'mgmt_driver': u'openwrt',
+            'attributes': {u'vnfd': self.tosca_vnfd_openwrt_param},
+            'id': u'fb048660-dc1b-4f0f-bd89-b023666650ec', 'name':
+            u'openwrt_services'}, 'mgmt_url': None, 'service_context': [],
+            'attributes': {'heat_template': utils.hot_data,
+                           'param_values': utils.update_new_param_data},
+            'id': 'eb84260e-5ff7-4332-b032-50a14d6c1123', 'description':
+                u'OpenWRT with services'}
+
     def _get_expected_active_vnf(self):
         return {'status': 'ACTIVE',
                 'instance_id': None,
@@ -154,7 +252,7 @@ class TestOpenStack(base.TestCase):
                     'attributes': {u'vnfd': self.tosca_vnfd_openwrt},
                     'id': u'fb048660-dc1b-4f0f-bd89-b023666650ec',
                     'name': u'openwrt_services'},
-                'mgmt_url': '{"vdu1": "192.168.120.31"}',
+                'mgmt_ip_address': '{"vdu1": "192.168.120.31"}',
                 'service_context': [],
                 'id': 'eb84260e-5ff7-4332-b032-50a14d6c1123',
                 'description': u'OpenWRT with services'}
@@ -181,6 +279,70 @@ class TestOpenStack(base.TestCase):
             'config'])
         self.assertEqual(expected_vnf_update, vnf_obj)
 
+    @mock.patch(
+        'tacker.vnfm.infra_drivers.openstack.vdu.Vdu')
+    def test_heal_vdu(self, mock_vdu):
+        self.infra_driver.heal_vdu(None, self.context, vnf_dict,
+                                   mock.ANY)
+        mock_vdu.assert_called_once_with(self.context, vnf_dict,
+                                         mock.ANY)
+
+    @mock.patch(
+        'tacker.vnfm.infra_drivers.openstack.vdu.Vdu')
+    @mock.patch('tacker.vnfm.infra_drivers.openstack.openstack.LOG')
+    def test_heal_vdu_failed(self, mock_log, mock_vdu):
+        mock_vdu.side_effect = Exception
+        self.assertRaises(vnfm.VNFHealFailed, self.infra_driver.heal_vdu,
+                          None, self.context, vnf_dict,
+                          mock.ANY)
+        mock_log.error.assert_called_with(
+            "VNF '%s' failed to heal", vnf_dict['id'])
+
+    def test_update_new_param(self):
+        vnf_obj = utils.get_dummy_vnf_param_attr()
+        vnf_param_obj = utils.get_dummy_vnf_update_new_param()
+        expected_vnf_update = self._get_expected_vnf_update_new_param_obj()
+        vnf_id = '4a4c2d44-8a52-4895-9a75-9d1c76c3e738'
+        self.infra_driver.update(plugin=None, context=self.context,
+                                 vnf_id=vnf_id, vnf_dict=vnf_obj,
+                                 vnf=vnf_param_obj,
+                                 auth_attr=utils.get_vim_auth_obj())
+        expected_vnf_update['attributes']['param_values'] = yaml.safe_load(
+            expected_vnf_update['attributes']['param_values'])
+        vnf_obj['attributes']['param_values'] = yaml.safe_load(
+            vnf_obj['attributes']['param_values'])
+        self.assertEqual(expected_vnf_update, vnf_obj)
+
+    @mock.patch('tacker.vnfm.infra_drivers.openstack.openstack.LOG')
+    def test_update_invalid_param(self, mock_log):
+        vnf_obj = utils.get_dummy_vnf_param_attr()
+        vnf_param_obj = utils.get_dummy_vnf_update_invalid_param()
+        vnf_id = '4a4c2d44-8a52-4895-9a75-9d1c76c3e738'
+        self.assertRaises(vnfm.VNFUpdateInvalidInput,
+                          self.infra_driver.update,
+                          plugin=None, context=self.context,
+                          vnf_id=vnf_id, vnf_dict=vnf_obj,
+                          vnf=vnf_param_obj,
+                          auth_attr=utils.get_vim_auth_obj())
+        log_msg = "at vnf_id {} because all parameters "\
+                  "match the existing one.".format(vnf_id)
+        mock_log.warning.assert_called_with(log_msg)
+
+    @mock.patch('tacker.vnfm.infra_drivers.openstack.openstack.LOG')
+    def test_update_empty_param(self, mock_log):
+        vnf_obj = utils.get_dummy_vnf_param_attr()
+        vnf_param_obj = utils.get_dummy_vnf_update_empty_param()
+        vnf_id = '4a4c2d44-8a52-4895-9a75-9d1c76c3e738'
+        self.assertRaises(vnfm.VNFUpdateInvalidInput,
+                          self.infra_driver.update,
+                          plugin=None, context=self.context,
+                          vnf_id=vnf_id, vnf_dict=vnf_obj,
+                          vnf=vnf_param_obj,
+                          auth_attr=utils.get_vim_auth_obj())
+        log_msg = "at vnf_id {} because the target "\
+                  "yaml is empty.".format(vnf_id)
+        mock_log.warning.assert_called_with(log_msg)
+
     def _get_expected_fields_tosca(self, template):
         return {'stack_name':
                 'test_openwrt_eb84260e-5ff7-4332-b032-50a14d6c1123',
@@ -204,7 +366,7 @@ class TestOpenStack(base.TestCase):
             },
             'id': 'eb84260e-5ff7-4332-b032-50a14d6c1123',
             'instance_id': None,
-            'mgmt_url': None,
+            'mgmt_ip_address': None,
             'name': u'test_openwrt',
             'service_context': [],
             'status': 'PENDING_CREATE',
@@ -215,36 +377,38 @@ class TestOpenStack(base.TestCase):
         if is_monitor:
             if multi_vdus:
                 dvc['attributes'].update(
-                    {'monitoring_policy': '{"vdus": {"VDU1": {"ping": '
-                                          '{"name": "ping", "actions": '
-                                          '{"failure": "respawn"}, '
-                                          '"parameters": {"count": 3, '
-                                          '"interval": 10}, '
-                                          '"monitoring_params": '
-                                          '{"count": 3, "interval": 10}}}, '
-                                          '"VDU2": {"ping": {"name": "ping", '
-                                          '"actions": {"failure": "respawn"}, '
-                                          '"parameters": {"count": 3, '
-                                          '"interval": 10}, '
-                                          '"monitoring_params": {"count": 3, '
-                                          '"interval": 10}}}}}'})
+                    {'monitoring_policy': helpers.compact_byte(
+                     '{"vdus": {"VDU1": {"ping": '
+                     '{"name": "ping", "actions": '
+                     '{"failure": "respawn"}, '
+                     '"parameters": {"count": 3, '
+                     '"interval": 10}, '
+                     '"monitoring_params": '
+                     '{"count": 3, "interval": 10}}}, '
+                     '"VDU2": {"ping": {"name": "ping", '
+                     '"actions": {"failure": "respawn"}, '
+                     '"parameters": {"count": 3, '
+                     '"interval": 10}, '
+                     '"monitoring_params": {"count": 3, '
+                     '"interval": 10}}}}}')})
             else:
                 dvc['attributes'].update(
-                    {'monitoring_policy': '{"vdus": {"VDU1": {"ping": '
-                                          '{"name": "ping", "actions": '
-                                          '{"failure": "respawn"}, '
-                                          '"parameters": {"count": 3, '
-                                          '"interval": 10}, '
-                                          '"monitoring_params": '
-                                          '{"count": 3, '
-                                          '"interval": 10}}}}}'})
+                    {'monitoring_policy': helpers.compact_byte(
+                     '{"vdus": {"VDU1": {"ping": '
+                     '{"name": "ping", "actions": '
+                     '{"failure": "respawn"}, '
+                     '"parameters": {"count": 3, '
+                     '"interval": 10}, '
+                     '"monitoring_params": '
+                     '{"count": 3, '
+                     '"interval": 10}}}}}')})
 
         return dvc
 
     def _get_dummy_tosca_vnf(self, template, input_params=''):
 
         tosca_template = _get_template(template)
-        vnf = utils.get_dummy_device_obj()
+        vnf = utils.get_dummy_vnf()
         dtemplate = self._get_expected_vnfd(tosca_template)
 
         vnf['vnfd'] = dtemplate
@@ -298,7 +462,7 @@ class TestOpenStack(base.TestCase):
                     vnf["attributes"][k])
             expected_vnf["attributes"]['scaling_group_names'] = {
                 'SP1': 'SP1_group'}
-            vnf["attributes"]['scaling_group_names'] = json.loads(
+            vnf["attributes"]['scaling_group_names'] = jsonutils.loads(
                 vnf["attributes"]['scaling_group_names']
             )
         self.assertEqual(expected_vnf, vnf)
@@ -322,6 +486,26 @@ class TestOpenStack(base.TestCase):
             'test_tosca_flavor_defaults.yaml',
             'hot_flavor_defaults.yaml')
 
+    def test_create_tosca_with_placement_policy_anti_affinity(self):
+        self._test_assert_equal_for_tosca_templates(
+            'tosca_placement_policy_anti_affinity.yaml',
+            'hot_placement_policy_anti_affinity.yaml', is_monitor=False)
+
+    def test_create_tosca_with_placement_policy_affinity(self):
+        self._test_assert_equal_for_tosca_templates(
+            'tosca_placement_policy_affinity.yaml',
+            'hot_placement_policy_affinity.yaml', is_monitor=False)
+
+    def test_create_tosca_with_placement_policy_soft_anti_affinity(self):
+        self._test_assert_equal_for_tosca_templates(
+            'tosca_placement_policy_soft_anti_affinity.yaml',
+            'hot_placement_policy_soft_anti_affinity.yaml', is_monitor=False)
+
+    def test_create_tosca_with_placement_policy_default_affinity(self):
+        self._test_assert_equal_for_tosca_templates(
+            'tosca_placement_policy_default_affinity.yaml',
+            'hot_placement_policy_default_affinity.yaml', is_monitor=False)
+
     def test_create_tosca_with_flavor_and_capabilities(self):
         self._test_assert_equal_for_tosca_templates(
             'test_tosca_flavor_and_capabilities.yaml',
@@ -392,6 +576,16 @@ class TestOpenStack(base.TestCase):
             input_params
         )
 
+    def test_tosca_artifacts_image_params(self):
+        input_params = 'image_source: ' \
+                       'http://download.cirros-cloud.net/' \
+                       '0.4.0/cirros-0.4.0-x86_64-disk.img'
+        self._test_assert_equal_for_tosca_templates(
+            'tosca_artifacts_image_vnfd_params.yaml',
+            'hot_tosca_artifacts_image_vnfd_params.yaml',
+            input_params
+        )
+
     def test_create_tosca_scale(self):
         self._test_assert_equal_for_tosca_templates(
             'tosca_scale.yaml',
